(*************************************************************************
*     Asignatura:     Estructuras de Datos y de la Informacin
*     Proyecto:       FORMATEADOR DE TEXTOS ASCII
*
*     Mdulo:  TAD TipoLinea
*     Fichero:     (x)TAD     ( )Programa    ()Otro Tipo
*     Autor(es):   Diego Chaparro Gonzlez
*     Fecha:       9 / Junio / 1999
*
*     Descripcion:
*        Contiene las operaciones necesarias para manejar una lnea de
*        texto tales como insertar caracter, colocar el texto, etc.
************************************************************************)

UNIT TipLinea;

INTERFACE
USES crt, strings;

TYPE
    TLinea= string [72];
    TipoFormato= -7..7;

    TipoLinea= OBJECT

       FUNCTION InsertarFormato ((*in*) formato: TipoFormato;(*in*) aux: byte): string;
       (* POST: Inserta el cdigo correspondiente al formato en el tipo linea*)
       FUNCTION InsertarCaracter ((*in*) car: char): string;
       (* POST: Inserta un carcter en el tipo fichero. Si el carcter desborda
          la linea, se devuelve la ltima palabra para insertarla en la siguiente
          linea *)
       PROCEDURE BorrarCaracter ((*in*) tecla: char);
       (* POST: Borra un carcter, dependiendo de la tecla pulsada (suprimir,borrado) *)

       PROCEDURE CrearLineaVacia ((*in*) alin, long: byte);
       (* POST: Crea una lnea vaca *)

       (*PROCEDURE BorrarLinea;*)
       (* POST: Borra la lnea actual *)

       {PROCEDURE PosicionarCursor;}
       (* POST: Coloca el cursor en el lugar correspondiente cuando se avanza
          o se retrocede lneas *)

       PROCEDURE Derecha;
       (* POST: Coloca el cursor una posicin a la derecha de la actual, dentro
          de una lnea *)

       PROCEDURE Izquierda;
       (* POST: Coloca el cursor una posicin a la izquierda de la actual, dentro
          de una lnea *)

       FUNCTION ActualCaracter: char;
       (* POST: Devuelve el caracter en el que se encuentra el cursor *)

       FUNCTION ActualPosicion: byte;
       (* POST: Devuelve la posicin actual en la lnea *)

       FUNCTION MaximoLinea: byte;
       (* POST: Devuelve el tamao mximo de la lnea *)

       FUNCTION DevolverLinea: STRING;
       (* POST: Devuelve el string actual *)

       PROCEDURE ColocarLinea;
       (* POST: Coloca la lnea segn el ajuste definido *)

       PROCEDURE DefinirFormato ((*in*) formato: TipoFormato; (*in*) aux: byte);
       (* POST: Define el formato adecuado para la lnea *)

       FUNCTION  InsertarPalabraPrincipio ((*in*) palabra: string):string;
       (* POST: Inserta una palabra al principio de la lnea *)

       FUNCTION DevolverColocacion: byte;
       FUNCTION LongitudActual: byte;
       FUNCTION DevolverNegrita: boolean;

       PRIVATE
              linea: TLinea;
              longitud: byte;
              posicion: byte;
              negrita: boolean;
              colocacion: byte; { colocacion=2 -> ajuste a dos mrgenes,
                                  colocacion=3 -> ajuste centrado,
                                  colocacion=4 -> ajuste izquierda,
                                  colocacion=5 -> ajuste derecha }
       END;

IMPLEMENTATION

      FUNCTION TipoLinea.InsertarFormato ((*in*) formato:TipoFormato;(*in*) aux: byte):string;
      (* Complejidad -- O(length(forma)) *)
      VAR
         forma, forma2: string;
         auxc: string;
         bucle,num, cont: byte;
      BEGIN
           Str (aux,auxc);
           CASE formato of { Elige el formato adecuado }
                1: forma:='#N#';
                -1: forma:= '#fN#';
                2: forma:='#H#';
                -2: forma:= '#fH#';
                3: forma:='#C#';
                -3: forma:= '#fC#';
                4: forma:='#I#';
                -4: forma:= '#fI#';
                5: forma:='#D#';
                -5: forma:= '#fD#';
                6: forma:='#E'+auxc+'#';
                -6: forma:= '#fE'+auxc+'#';
                7: forma:='#L'+auxc+'#';
                -7: forma:= '#fL'+auxc+'#';
           END;
           IF ((length(forma)+posicion) <= longitud) AND (length(linea)+length(forma)<longitud) THEN
           BEGIN
                Insert (forma,linea,posicion);
                posicion:=posicion+length(forma);
                InsertarFormato:= '';
           END
           ELSE
           BEGIN
                forma2:='';
                IF ((posicion+length(forma)) <= longitud) THEN
                BEGIN
                     cont:=longitud;
                     WHILE (linea[cont]<>' ') OR (cont+length(forma)>longitud) DO
                     {mirar la longitud de la palabra que no cabe en la frase}
                           cont:=cont-1;
                     forma2:= copy (linea, cont, (longitud-cont+1));
                     delete (linea, cont+1, (longitud-cont)); { Borrar la palabra que no cabe}

                     IF cont>=posicion THEN
                     BEGIN
                          Insert (forma,linea,posicion);
                          posicion:= posicion+length(forma);
                     END
                     ELSE
                     BEGIN
                         Insert (forma,linea,cont);
                         posicion:= cont;
                     END;
                END
                ELSE
                BEGIN

                     IF posicion<>length(linea)+1 THEN
                     BEGIN
                          WHILE (linea[cont]<>' ') OR (cont+length(forma)>longitud) DO
                          {mirar la longitud de la palabra que no cabe en la frase}
                                cont:=cont-1;
                          forma2:= copy (linea, cont+1, (length(linea)-cont));
                          delete (linea, cont, (length(linea)-cont)+1); { Borrar la palabra que no cabe}
                          posicion:=cont;
                          Insert (forma, forma2, 1);
                     END
                     ELSE
                         forma2:=forma;
                END;
                InsertarFormato:= forma2;
           END;
           IF forma[length(forma)]=' ' THEN Delete (forma,length(forma),1);

      END;

      FUNCTION TipoLinea.InsertarCaracter ((*in*) car: char): string;
      VAR
         forma, forma2: string;
         cont, bucle: byte;
         sw: boolean;
      BEGIN
           IF ((posicion+1) <= longitud) AND (length(linea)<longitud) THEN
           BEGIN
                Insert (car,linea,posicion);
                posicion:=posicion+1;
                InsertarCaracter:= '';
           END
           ELSE
           BEGIN
                forma:='';
                IF (posicion<longitud) THEN
                BEGIN
                     cont:=longitud;
                     WHILE (linea[cont]<>' ') OR (cont+1>longitud) DO {mirar la longitud de la palabra que no cabe en la frase}
                           cont:=cont-1;
                     forma:= copy (linea, cont, (longitud-cont+1));
                     delete (linea, cont+1, (longitud-cont)); { Borrar la palabra que no cabe}

                     IF cont>=posicion THEN
                     BEGIN
                          Insert (car,linea,posicion);
                          posicion:= posicion+1;
                     END
                     ELSE
                     BEGIN
                         Insert (car,forma,posicion-cont);
                         posicion:= cont;
                     END;
                END
                ELSE
                BEGIN
                     cont:=posicion-1;
                     IF car<>' ' THEN { Si el carcter insertado no es un blanco }
                     BEGIN
                          WHILE ((linea[cont]<>' ') OR (cont+1>longitud)) AND (cont>1)  DO
                          {mirar la longitud de la palabra que no cabe en la frase}
                                cont:=cont-1;
                             forma:= copy (linea, cont+1, (length(linea)-cont));
                             delete (linea, cont, (length(linea)-cont)+1); { Borrar la palabra que no cabe}
                          posicion:=cont;
                          forma:=forma+car;
                     END
                     ELSE
                         forma:=' ';
                END;
                InsertarCaracter:= forma;
           END;
           IF forma[length(forma)]=' ' THEN Delete (forma,length(forma),1);

      END;


      PROCEDURE TipoLinea.BorrarCaracter ((*in*) tecla: char);
      BEGIN
           IF ((tecla='1') AND (linea[posicion] <> '')) THEN {Suprimir}
           BEGIN
                delete (linea,posicion,1);
           END
           ELSE
           BEGIN  { Borrar }
             IF (tecla='2') THEN
                IF ((posicion>=2) AND (linea[posicion-1] <> '')) THEN
                BEGIN
                     posicion := posicion - 1;
                     delete (linea,posicion,1);
                END;
           END;
      END;

      PROCEDURE TipoLinea.CrearLineaVacia ((*in*) alin, long: byte);
      BEGIN
           linea:='';
           posicion:=1;
           longitud:=long;
           colocacion:=alin;
           negrita:= false;
      END;

      FUNCTION TipoLinea.ActualCaracter: char;
      BEGIN
           ActualCaracter:=linea[posicion];
      END;


      FUNCTION TipoLinea.ActualPosicion: byte;
      BEGIN
           ActualPosicion:= posicion;
      END;

      FUNCTION TipoLinea.MaximoLinea: byte;
      BEGIN
           MaximoLinea:= longitud;
      END;

      PROCEDURE TipoLinea.Izquierda;
      BEGIN
           IF posicion>1 THEN
              posicion:= posicion-1;
      END;

      PROCEDURE TipoLinea.Derecha;
      BEGIN
           IF posicion<length(linea)+1 THEN
              posicion:= posicion+1;
      END;

      FUNCTION TipoLinea.DevolverLinea: string;
      BEGIN
           DevolverLinea:= linea;
      END;

      PROCEDURE TipoLinea.ColocarLinea;
      VAR
         cont,num,bucle: byte;
         i, j : byte;
      BEGIN
           CASE colocacion OF

           2: {ambos}
           BEGIN
              IF linea[length(linea)] <> '' THEN
              BEGIN
                cont := longitud - length(linea);
                i := 0;
                WHILE i < cont DO
                BEGIN
                    j := 1;
                    WHILE ((j <= length(linea)) AND (i < cont)) DO
                    BEGIN
                         IF linea[j] = ' ' THEN
                         BEGIN
                            Insert(' ',linea, j);
                            i := i + 1;
                            j := j + 1;
                         END;
                         j := j + 1;
                    END;
                END;
                posicion := length(linea) + 1;
              END;
           END;
           3: {centrado}
           BEGIN
                cont:=0;
                bucle:=1;
                WHILE (cont=0) AND (bucle<=length(linea)) DO
                BEGIN
                    IF linea[bucle]<>' ' THEN cont:=bucle;
                    bucle:=bucle+1;
                END;
                num:=(longitud-(length(linea)-cont)) DIV 2;
                FOR bucle:=1 TO num DO
                    Insert (' ',linea,1);
                posicion := length(linea) + 1;
           END;
           4: {izquierda}
           BEGIN
             WHILE linea[1]=' ' DO Delete (linea,1,1);
             posicion := length(linea) + 1;
           END;

           5: {derecha}
           BEGIN
                num:=(longitud-length(linea));
                FOR bucle:=1 to num DO
                    Insert (' ',linea,1);
                posicion := length(linea) + 1;
           END;
           END;
      END;

      PROCEDURE TipoLinea.DefinirFormato ((*in*) formato: TipoFormato;
                (*in*) aux: byte);
      BEGIN
           CASE formato OF
                -1: negrita:= false;
                -2: colocacion:= aux;
                -3: colocacion:= aux;
                -4: colocacion:= aux;
                -5: colocacion:= aux;
                -7: longitud:=aux;
                1: negrita:= true;
                2: colocacion:=2;
                3: colocacion:=3;
                4: colocacion:=4;
                5: colocacion:=5;
                7: longitud:= aux;
           END;
      END;

      FUNCTION TipoLinea.InsertarPalabraPrincipio ((*in*) palabra: string):string;
      VAR
         pal,paltotal: string;
         cont: byte;
      BEGIN
           paltotal:='';
           IF length(linea)=0 THEN
           BEGIN
                Insert (palabra,linea,1);
                palabra:='';
           END;
           IF length (palabra)+length(linea)>longitud THEN
           BEGIN
                pal:='';
                cont:=length(linea);
                WHILE (linea[cont]<>' ') OR (cont>longitud) DO {mirar la longitud de la palabra que no cabe en la frase}
                      cont:=cont-1;
                pal:= copy (linea, cont+1, (length(linea)-cont));
                delete (linea, cont, (length(linea)-cont+1)); { Borrar la palabra que no cabe}
                IF length(paltotal) <>0 THEN paltotal:=' '+paltotal;
                Insert (pal,paltotal,1);
           END;
           IF (length(linea)>0) AND (palabra<>'')THEN
              palabra:=palabra+' ';
           IF palabra<>'' THEN
           BEGIN
                Insert (palabra,linea,1);
                posicion:=length (palabra)+1;
           END
           ELSE
               posicion:=length(linea)+1;
           InsertarPalabraPrincipio:= paltotal;
      END;

      FUNCTION TipoLinea.LongitudActual:byte;
      BEGIN
           LongitudActual:=length (linea);
      END;

      FUNCTION TipoLinea.DevolverColocacion: byte;
      BEGIN
           DevolverColocacion:=colocacion;
      END;
      FUNCTION TipoLinea.DevolverNegrita: boolean;
      BEGIN
           DevolverNegrita:= negrita;
      END;

END.